# Continuous Integration Tools

These scripts are used to implement Continuous Integration on Travis, AppVeyor
and Azure.  They should not be removed without careful consideration of the 
consequences.
